@@ -12,7 +12,9 @@ import           Data.List
 import           Control.Monad (when)
 import           System.Exit (exitFailure)
 import           System.IO
+import           System.Environment (getEnvironment)
 
+import           Control.Applicative
 import qualified Control.Exception as E
 import           Panic
 
@@ -21,6 +23,13 @@ import           Help
 import           Report
 import qualified Interpreter
 
+ghcPackageDbFlag :: String
+#if __GLASGOW_HASKELL__ >= 706
+ghcPackageDbFlag = "-package-db"
+#else
+ghcPackageDbFlag = "-package-conf"
+#endif
+
 -- | Run doctest with given list of arguments.
 --
 -- Example:
@@ -35,17 +44,26 @@ doctest args
   | "--help"    `elem` args = putStr usage
   | "--version" `elem` args = printVersion
   | otherwise = do
+      -- Look up the HASKELL_PACKAGE_SANDBOX environment variable and, if
+      -- present, add it to the list of package databases GHC searches.
+      -- Intended to make testing from inside sandboxes such as cabal-dev
+      -- simpler.
+      packageConf <- lookup "HASKELL_PACKAGE_SANDBOX" <$> getEnvironment
+      let addPackageConf = case packageConf of
+            Nothing -> id
+            Just p  -> \rest -> ghcPackageDbFlag : p : rest
+
       let (f, args_) = stripOptGhc args
       when f $ do
         hPutStrLn stderr "WARNING: --optghc is deprecated, doctest now accepts arbitrary GHC options\ndirectly."
         hFlush stderr
-      r <- doctest_ args_ `E.catch` \e -> do
+      r <- doctest_ (addPackageConf args_) `E.catch` \e -> do
         case fromException e of
           Just (UsageError err) -> do
             hPutStrLn stderr ("doctest: " ++ err)
             hPutStrLn stderr "Try `doctest --help' for more information."
             exitFailure
-          _ -> E.throw e
+          _ -> E.throwIO e
       when (not $ isSuccess r) exitFailure
 
 isSuccess :: Summary -> Bool
